from typing import Optional
from datetime import datetime


class DateFilter:
    def __init__(self,
                 start: Optional[datetime] = None,
                 end: Optional[datetime] = None):
        self.start = start
        self.end = end

    def to_api_param(self):
        params = ""
        if self.start:
            params += f"&start_date={self.start.isoformat()}"
        if self.end:
            params += f"&end_date={self.end.isoformat()}"
        return params



import pandas as pd
import numpy as np
from datetime import datetime
from typing import Optional
from .parameters.dates import DateFilter
from .api.base import DefaultAPI, APIInterface


class GSN:
    def __init__(self, api: APIInterface):
        self.api = api

    def get_sites(self) -> pd.DataFrame:
        return pd.read_json(
            self.api.sites(),
            dtype={
                "id": np.uint,
                "name": str,
                "description": str,
                "location": str,
                "insar_available": np.bool,
                "owner": str,
                "num_sensors": np.uint
            }
        )

    def get_site(self, site_id: str) -> pd.DataFrame:
        return pd.read_json(
            self.api.sites(site_id),
            dtype={
                "id": np.uint,
                "name": str,
                "description": str,
                "location": str,
                "insar_available": np.bool,
                "owner": str,
                "num_sensors": np.uint
            }
        )

    def get_raw_sp_for_site(self,
                            site_id: str,
                            start: Optional[datetime] = None,
                            end: Optional[datetime] = None) -> pd.DataFrame:
        return pd.read_json(
            self.api.sp(site_id, DateFilter(start, end)),
            dtype={
                "pot_number": np.uint,
                "time": np.datetime64,
                "potential": np.single,
                "units": str,
                "pot_notes": Optional[str],
                "site_id": np.uint
            },
            convert_dates=["time", ]
        )


def connect_to_gsn(url: str, code: str, api=DefaultAPI) -> GSN:
    return GSN(api(url, code))



import abc
import requests as rq
from typing import Optional
from ..parameters.dates import DateFilter


class APIInterface(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def sites(self, site_id: Optional[str] = None) -> str:
        raise NotImplementedError("Define the 'sites' method of the API interface.")

    @abc.abstractmethod
    def sp(self, site_id: str, date_filter: DateFilter) -> str:
        raise NotImplementedError("Define the 'sp' method of the API interface.")


class DefaultAPI(APIInterface):
    def __init__(self, url: str, code: str):
        self.url = url
        self.code = code

    def sites(self, site_id: Optional[str] = None) -> str:
        if site_id:
            return rq.get(f"{self.url}/sites/{site_id}?{self._code_string()}").text
        else:
            return rq.get(f"{self.url}/sites?{self._code_string()}").text

    def sp(self, site_id: str, date_filter: DateFilter) -> str:
        params = self._code_string() + date_filter.to_api_param()
        return rq.get(f"{self.url}/sp/{site_id}?{params}").text

    def _code_string(self):
        return f"code={self.code}"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="spivolt",
    version="0.0.2",
    author="Nick Farnham",
    author_email="nick.farnham@southwales.ac.uk",
    description="A wrapper for the SPiVolt API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/brythonick/spivolt",
    packages=setuptools.find_packages(include=["spivolt", "spivolt.*"], exclude=["tests", ]),
    install_requires=[
        "pandas",
        "requests",
        "numpy",
        "matplotlib"
    ],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Scientific/Engineering",
    ],
    python_requires='>=3.6',
)



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime
from typing import Optional


@pd.api.extensions.register_dataframe_accessor("sp")
class SpAccessor:
    def __init__(self, pandas_obj: pd.DataFrame):
        self._validate(pandas_obj)
        self._obj = pandas_obj.sort_values(["time", "pot_number"])

    @staticmethod
    def _validate(obj):
        if "potential" not in obj.columns:
            raise AttributeError("Must have 'potential' field.")
        elif "pot_number" not in obj.columns:
            raise AttributeError("Must have 'pot_number' field.")

    def between(self, start: Optional[datetime] = None, end: Optional[datetime] = None) -> pd.DataFrame:
        data = self._obj.copy()
        if start:
            data = data[data["time"] >= start]
        if end:
            data = data[data["time"] <= end]
        return data

    def potwise(self) -> pd.DataFrame:
        return self._obj.pivot_table(index="time", columns="pot_number", values="potential")

    def plot(self, figsize=(13, 50)):
        sensor_list = self._obj.pot_number.unique()
        fig, ax = plt.subplots(nrows=len(sensor_list), ncols=1, figsize=figsize)
        for current_pot in sensor_list:
            data = self._obj[self._obj.pot_number == current_pot]
            idx = np.where(sensor_list == current_pot)[0][0]
            ax[idx].plot(data.time, data.potential)
            ax[idx].set_title(f"SP: Pot {current_pot}")
            ax[idx].set_ylabel("Potential (mV)")
        fig.tight_layout()
